import random


def colorize(speaker):
    # Set the seed based on the peer name
    random.seed(speaker)

    # Generate random RGB values for lighter colors
    r = random.randint(0, 255)
    g = random.randint(0, 255)
    b = random.randint(0, 255)

    # Generate the color code
    color_code = "#{:02X}{:02X}{:02X}".format(r, g, b)

    return color_code



import tkinter as tk

from .broadcast_message_entry import BroadcastMessageEntry
from .peers import Peers
from .broadcast_view import BroadcastView


class BroadcastTab(tk.Frame):
    def __init__(self, root, app):
        super().__init__(root)
        self.app = app
        self.toolbar_frame = tk.Frame(self, background="#333232")
        self.toolbar_frame.pack(side=tk.TOP, fill=tk.X)
        self.messages_and_peers_frame = tk.Frame(self)
        self.messages_and_peers_frame.pack(fill=tk.BOTH, expand=True)
        self.peers = Peers(self.messages_and_peers_frame, app)
        self.message_table = BroadcastView(self.messages_and_peers_frame, app, self)
        self.more_link = tk.Label(
            self.toolbar_frame,
            text=self.message_table.page_monitor.button_label(),
            fg="#00a1ff",
            cursor="hand2",
            background="#333232",
        )
        self.message_table.await_message_stats()
        self.more_link.pack(side=tk.LEFT)
        self.more_link.bind("<Button-1>", lambda e: self.message_table.load_previous())
        self.message_entry = BroadcastMessageEntry(self, app)
        self.app.register_message_listener(self.message_table)
        self.app.register_message_listener(self.peers)



import datetime
import time


class PageMonitor:
    def __init__(self):
        self.days_before = 1

    def in_window(self, message):
        now = time.time()
        ts = message.get("timestamp")
        if (
            ts
            >= (
                datetime.datetime.fromtimestamp(now)
                - datetime.timedelta(days=self.days_before)
            ).timestamp()
        ):
            return True

    def button_label(self):
        return f"《 prev {self.days_before + 1} days"



import tkinter as tk


class Peers(tk.Frame):
    def __init__(self, root, app):
        self.app = app
        self.frame = tk.Frame(
            root, borderwidth=0, highlightthickness=0, width=200, background="black"
        )
        self.frame.pack(side=tk.RIGHT, fill=tk.Y)

        # peer list
        self.peer_list = tk.Text(
            self.frame, highlightthickness=0, borderwidth=0, width=20
        )
        self.peer_list.configure(background="black", foreground="white")
        self.peer_list.pack(fill=tk.BOTH, side=tk.BOTTOM, expand=True, padx=10, pady=10)
        self.peer_list.config(state="disabled")
        self.peer_list.config(cursor="arrow")
        self.peer_list.tag_config("offline", font=("", "12", "italic"))
        self.peer_list.tag_config("online", font=("", "12"))
        self.peer_list.tag_bind("clickable", "<Button-1>", self.on_peer_click)
        self.peer_list.tag_bind(
            "clickable", "<Enter>", (lambda e: self.peer_list.config(cursor="hand2"))
        )
        self.peer_list.tag_bind(
            "clickable", "<Leave>", (lambda e: self.peer_list.config(cursor="xterm"))
        )

    def on_peer_click(self, event):
        # Get the index of the mouse pointer
        index = self.peer_list.index("@%s,%s" % (event.x, event.y))

        # Get the line number
        line_number = index.split(".")[0]

        # Get the line text
        handle = self.peer_list.get(f"{line_number}.0", f"{line_number}.end")
        print("clicked on peer: %s" % handle)
        self.app.open_direct_message_tab(handle)

    def render_message(self, message):
        self.peer_list.config(state="normal")
        if message["command"] == "presence":
            handle = message["handle"]
            if not handle in self.peer_list.get("1.0", tk.END).splitlines():
                self.peer_list.insert(tk.END, handle + "\n")
                self.tag_peer(handle, "clickable")
            if message["type"] == "online":
                self.tag_peer(handle, "online")
            elif message["type"] == "offline":
                self.tag_peer(handle, "offline")
            else:
                self.remove_peer(handle)
        self.peer_list.config(state="disabled")

    def tag_peer(self, handle, tag):
        peer_index = self.get_peer_index(handle)
        if peer_index:
            if tag == "offline":
                self.peer_list.tag_remove("online", peer_index, peer_index + "+1l")
                self.peer_list.tag_add("offline", peer_index, peer_index + "+1l")
            elif tag == "online":
                self.peer_list.tag_remove("offline", peer_index, peer_index + "+1l")
                self.peer_list.tag_add("online", peer_index, peer_index + "+1l")
            else:
                self.peer_list.tag_add(tag, peer_index, peer_index + "+1l")

    def get_peer_index(self, handle):
        for i, peer in enumerate(self.get_peers()):
            if peer == handle:
                return str(float(i) + 1.0)
        return None

    def get_peers(self):
        peers = self.peer_list.get("1.0", tk.END).splitlines()
        if len(peers) > 0:
            return peers[:-1]
        return []

    def remove_peer(self, handle):
        peer_index = self.get_peer_index(handle)
        self.peer_list.delete(peer_index, peer_index + "+1l")



import hashlib
import random
import sqlite3
import string
import time
import json

from akris_desktop.app import App
from locust.client import Client
from unittest.mock import MagicMock
from queue import Queue
import tkinter as tk


# def test_populate_peers():
#     root = tk.Tk()
#     client = MagicMock(Client)
#     client.message_queue = Queue()
#     for peer in generate_peers():
#         client.message_queue.put(peer)
#     app = App(root, client)
#     app.run()
#     root.mainloop()


def generate_peers():
    return [
        {
            "command": "peer",
            "status": "online",
            "handle": "asciilifeform",
        },
        {
            "command": "peer",
            "status": "online",
            "handle": "ben_vulpes",
        },
        {
            "command": "peer",
            "status": "online",
            "handle": "signpost",
        },
        {
            "command": "peer",
            "status": "online",
            "handle": "awt",
        },
    ]



from treelib import Node, Tree


class Chain:
    def __init__(self):
        self.chain = Tree()
        self.chain.create_node("root", "root")

    def add_message(self, message):
        # if the tree is empty, add the message as a child of root
        if self.chain.children("root") == []:
            self.chain.create_node(
                message["message_hash"],
                message["message_hash"],
                parent="root",
                data=message,
            )
            return

        # if the message is an antecedent (message_hash == net_chain), insert it at the top and reassign the children
        # search for descendants of the message
        def filter_function(node):
            if node.data:
                return node.data["net_chain"] == message["message_hash"]
            else:
                return False

        descendants = list(self.chain.filter_nodes(filter_function))
        if descendants:
            # insert the message at the top of the chain
            self.chain.create_node(
                message["message_hash"],
                message["message_hash"],
                parent="root",
                data=message,
            )
            # reassign the children
            for node in descendants:
                self.chain.move_node(node.identifier, message["message_hash"])

        # if the message is a descendant of a preexisting message, add it to the descendants
        else:
            self.chain.create_node(
                message["message_hash"],
                message["message_hash"],
                parent=message["net_chain"],
                data=message,
            )

    def index(self, message_hash):
        expansion = self.chain.expand_tree(
            mode=Tree.WIDTH, sorting=True, key=self.sort_by_timestamp
        )
        for index, id in enumerate(expansion):
            if id == message_hash:
                return self.indexize(index)

    def sort_by_timestamp(self, node):
        timestamp = node.data["timestamp"]
        return timestamp

    def indexize(self, index):
        return str(float(index))



import tkinter as tk


class Help:
    def __init__(self, output):
        self.output = output

    def render(self, response):
        help_dict = response.get("body")
        for key, value in help_dict.items():
            self.render_help(value["help"])

    def render_help(self, error):
        self.output.configure(state="normal")
        self.output.insert(tk.END, "{}\n".format(error))
        self.output.configure(state="disabled")



from setuptools import setup, find_packages

setup(
    name="akris-desktop",
    version="1.0.1",
    description="A gui client for akris",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Adam Thorsen",
    author_email="awt@fastmail.fm",
    url="http://v.alethepedia.com/akris_desktop",
    license="VPL",
    packages=find_packages(),
    install_requires=["Pillow", "akris"],
    extras_require={
        "dev": [
            "black",
            "pytest",
            "pytest-mock",
            "autopep8",
            "pyinstaller",
            "markdown2",
            "staticx",
            "pylint",
        ]
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
    ],
    keywords="p2p pest forum",
    python_requires=">=3.11",
    entry_points={
        "console_scripts": [
            "akris-desktop=akris_desktop.main:main",
        ],
    },
)



"""
Emoji database.
"""

__all__ = [
    "Emoji",
    "get_emoji_aliases",
    "get_emoji_by_code",
    "get_emoji_by_alias",
    "get_emojis_by_tag",
    "get_emojis_by_category",
    "get_tags",
    "get_categories",
]

from .db import Emoji
from .utils import (
    get_emoji_aliases,
    get_emoji_by_code,
    get_emoji_by_alias,
    get_emojis_by_tag,
    get_emojis_by_category,
    get_tags,
    get_categories,
)



class NewHandleListener:
    def __init__(self, app):
        self.app = app

    def render_message(self, message):
        if message.get("ref_link_page_response"):
            return

        if message["command"] in ["direct_text", "direct_text_m"]:
            handle = message.get("handle")
            if handle in self.app.broadcast_tab.peers.get_peers():
                if handle not in [
                    self.app.notebook.tab(t, "text") for t in self.app.direct_tabs
                ]:
                    self.app.add_direct_message_tab(handle)



from akris_desktop.timeline import Timeline, Annotation, BEFORE, AFTER

def test_add_message():
    timeline = Timeline()
    message = {
        "body": "test",
        "self_chain": "foo",
        "net_chain": "bar",
        "timestamp": 100
    }
    index = timeline.add_message(message)
    assert index == 1

def test_messages_ordered_by_chain():
    timeline = Timeline()
    m1 = {
        "body": "test",
        "self_chain": "foo",
        "net_chain": "bar",
        "timestamp": 100,
        "message_hash": "baz"
    }
    m2 = {
        "body": "test",
        "self_chain": "baz",
        "net_chain": "baz",
        "timestamp": 100,
        "message_hash": "xyz",
    }
    timeline.add_message(m1)
    index = timeline.add_message(m2)
    assert index == 2

def test_add_annotation():
    timeline = Timeline()
    message = {
        "body": "test",
        "self_chain": "foo",
        "net_chain": "bar",
        "timestamp": 100
    }
    timeline.add_message(message)
    annotation = Annotation(100, "test", BEFORE)
    index = timeline.add_annotation(annotation)
    assert index == 1

def test_annotated_message_index():
    timeline = Timeline()
    message = {
        "body": "test",
        "self_chain": "foo",
        "net_chain": "bar",
        "timestamp": 100
    }
    timeline.add_message(message)
    annotation = Annotation(100, "test", BEFORE)
    timeline.add_annotation(annotation)
    index = timeline.calculate_message_index(message)
    assert index == 2

def test_account_for_antecedent_timestamp_groups():
    timeline = Timeline()
    m1 = {
        "body": "test",
        "self_chain": "foo",
        "net_chain": "bar",
        "timestamp": 100,
        "message_hash": "baz"
    }
    m2 = {
        "body": "test",
        "self_chain": "baz",
        "net_chain": "baz",
        "timestamp": 100,
        "message_hash": "xyz",
    }
    m3 = {
        "body": "test",
        "self_chain": "baz",
        "net_chain": "baz",
        "timestamp": 300,
        "message_hash": "xyz",
    }
    timeline.add_message(m1)
    timeline.add_message(m2)
    timeline.add_message(m3)
    index = timeline.calculate_message_index(m3)
    assert index == 3

def test_account_for_newlines():
    timeline = Timeline()
    m1 = {
        "body": "test\n test\n test",
        "self_chain": "foo",
        "net_chain": "bar",
        "timestamp": 100,
        "message_hash": "baz"
    }
    m2 = {
        "body": "test",
        "self_chain": "baz",
        "net_chain": "baz",
        "timestamp": 100,
        "message_hash": "xyz",
    }
    timeline.add_message(m1)
    timeline.add_message(m2)
    index = timeline.calculate_message_index(m2)
    assert index == 4

def test_account_for_newlines_in_antecedent_ts_groups():
    timeline = Timeline()
    m1 = {
        "body": "test\n test\n test",
        "self_chain": "foo",
        "net_chain": "bar",
        "timestamp": 100,
        "message_hash": "baz"
    }
    m2 = {
        "body": "test",
        "self_chain": "baz",
        "net_chain": "baz",
        "timestamp": 200,
        "message_hash": "xyz",
    }
    timeline.add_message(m1)
    timeline.add_message(m2)
    index = timeline.calculate_message_index(m2)
    assert index == 4
